/*
 * Copyright (c) 2017 GigaDevice Semiconductor Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gd32f3x0_syscfg.h"

/*!
	\brief	  reset the SYSCFG registers
	\param[in]  none
	\param[out] none
	\retval	 none
*/
void syscfg_deinit(void)
{
	rcu_periph_reset_enable(RCU_CFGCMPRST);
	rcu_periph_reset_disable(RCU_CFGCMPRST);
}

/*!
	\brief	  enable the DMA channels remapping
	\param[in]  syscfg_dma_remap: specify the DMA channels to remap
	  \arg		SYSCFG_DMA_REMAP_TIMER16: remap TIMER16 channel0 and UP DMA requests to channel1(defaut channel0)
	  \arg		SYSCFG_DMA_REMAP_TIMER15: remap TIMER15 channel2 and UP DMA requests to channel3(defaut channel2)
	  \arg		SYSCFG_DMA_REMAP_USART0RX: remap USART0 Rx DMA request to channel4(default channel2)
	  \arg		SYSCFG_DMA_REMAP_USART0TX: remap USART0 Tx DMA request to channel3(default channel1)
	  \arg		SYSCFG_DMA_REMAP_ADC: remap ADC DMA requests from channel0 to channel1
	\param[out] none
	\retval	 none
*/
void syscfg_dma_remap_enable(uint32_t syscfg_dma_remap)
{
	SYSCFG_CFG0 |= syscfg_dma_remap;
}

/*!
	\brief	  disable the DMA channels remapping
	\param[in]  syscfg_dma_remap: specify the DMA channels to remap
	  \arg		SYSCFG_DMA_REMAP_TIMER16: remap TIMER16 channel0 and UP DMA requests to channel1(defaut channel0)
	  \arg		SYSCFG_DMA_REMAP_TIMER15: remap TIMER15 channel2 and UP DMA requests to channel3(defaut channel2)
	  \arg		SYSCFG_DMA_REMAP_USART0RX: remap USART0 Rx DMA request to channel4(default channel2)
	  \arg		SYSCFG_DMA_REMAP_USART0TX: remap USART0 Tx DMA request to channel3(default channel1)
	  \arg		SYSCFG_DMA_REMAP_ADC: remap ADC DMA requests from channel0 to channel1
	\param[out] none
	\retval	 none
*/
void syscfg_dma_remap_disable(uint32_t syscfg_dma_remap)
{
	SYSCFG_CFG0 &= ~syscfg_dma_remap;
}

/*!
	\brief	  enable PB9 high current capability
	\param[in]  none
	\param[out] none
	\retval	 none
*/
void syscfg_high_current_enable(void)
{
	SYSCFG_CFG0 |= SYSCFG_HIGH_CURRENT_ENABLE;
}

/*!
	\brief	  disable PB9 high current capability
	\param[in]  none
	\param[out] none
	\retval	 none
*/
void syscfg_high_current_disable(void)
{
	SYSCFG_CFG0 &= SYSCFG_HIGH_CURRENT_DISABLE;
}

/*!
	\brief	  configure the GPIO pin as EXTI Line
	\param[in]  exti_port: specify the GPIO port used in EXTI
	  \arg		EXTI_SOURCE_GPIOx(x = A,B,C,D,F): EXTI GPIO port
	\param[in]  exti_pin: specify the EXTI line
	  \arg		EXTI_SOURCE_PINx(x = 0..15): EXTI GPIO pin
	\param[out] none
	\retval	 none
*/
void syscfg_exti_line_config(uint8_t exti_port, uint8_t exti_pin)
{
	uint32_t clear_exti_mask = ~((uint32_t)EXTI_SS_MASK << (EXTI_SS_MSTEP(exti_pin)));
	uint32_t config_exti_mask = ((uint32_t)exti_port) << (EXTI_SS_MSTEP(exti_pin));

	switch(exti_pin / EXTI_SS_JSTEP){
	case EXTISS0:
		/* clear EXTI source line(0..3) */
		SYSCFG_EXTISS0 &= clear_exti_mask;
		/* configure EXTI soure line(0..3) */
		SYSCFG_EXTISS0 |= config_exti_mask;
		break;
	case EXTISS1:
		/* clear EXTI soure line(4..7) */
		SYSCFG_EXTISS1 &= clear_exti_mask;
		/* configure EXTI soure line(4..7) */
		SYSCFG_EXTISS1 |= config_exti_mask;
		break;
	case EXTISS2:
		/* clear EXTI soure line(8..11) */
		SYSCFG_EXTISS2 &= clear_exti_mask;
		/* configure EXTI soure line(8..11) */
		SYSCFG_EXTISS2 |= config_exti_mask;
		break;
	case EXTISS3:
		/* clear EXTI soure line(12..15) */
		SYSCFG_EXTISS3 &= clear_exti_mask;
		/* configure EXTI soure line(12..15) */
		SYSCFG_EXTISS3 |= config_exti_mask;
		break;
	default:
		break;
	}
}

/*!
	\brief	  connect TIMER0/14/15/16 break input to the selected parameter
	\param[in]  syscfg_lock: Specify the parameter to be connected
	  \arg		SYSCFG_LOCK_LOCKUP: Cortex-M4 lockup output connected to the break input
	  \arg		SYSCFG_LOCK_SRAM_PARITY_ERROR: SRAM_PARITY check error connected to the break input
	  \arg		SYSCFG_LOCK_LVD: LVD interrupt connected to the break input
	\param[out] none
	\retval	 none
*/
void syscfg_lock_config(uint32_t syscfg_lock)
{
	SYSCFG_CFG2 |= syscfg_lock;
}

/*!
	\brief	  check if the specified flag in SYSCFG_CFG2 is set or not.
	\param[in]  syscfg_flag: specify the flag in SYSCFG_CFG2 to check.
	  \arg		SYSCFG_SRAM_PCEF: SRAM parity check error flag.
	\param[out] none
	\retval	 the syscfg_flag state returned (SET or RESET).
  */
FlagStatus syscfg_flag_get(uint32_t syscfg_flag)
{
	if((SYSCFG_CFG2 & syscfg_flag) != (uint32_t)RESET){
		return SET;
	}else{
		return RESET;
	}
}

/*!
	\brief	  clear the flag in SYSCFG_CFG2 by writing 1.
	\param[in]  syscfg_flag: Specify the flag in SYSCFG_CFG2 to clear.
	  \arg		SYSCFG_SRAM_PCEF: SRAM parity check error flag.
	\param[out] none
	\retval	 none
*/
void syscfg_flag_clear(uint32_t syscfg_flag)
{
	SYSCFG_CFG2 |= (uint32_t) syscfg_flag;
}

/*!
	\brief	  configure the I/O compensation cell
	\param[in]  syscfg_compensation: specifies the I/O compensation cell mode
	  \arg		SYSCFG_COMPENSATION_ENABLE: I/O compensation cell is enabled
	  \arg		SYSCFG_COMPENSATION_DISABLE: I/O compensation cell is disabled
	\param[out] none
	\retval	 none
*/
void syscfg_compensation_config(uint32_t syscfg_compensation) 
{
	uint32_t reg;

	reg = SYSCFG_CPSCTL;
	/* reset the SYSCFG_CPSCTL_CPS_EN bit and set according to syscfg_compensation */
	reg &= ~SYSCFG_CPSCTL_CPS_EN;
	SYSCFG_CPSCTL = (reg | syscfg_compensation);
}

/*!
	\brief	  check if the I/O compensation cell ready flag is set or not
	\param[in]  none
	\param[out] none
	\retval	 FlagStatus: SET or RESET
  */
FlagStatus syscfg_cps_rdy_flag_get(void)
{
	if(((uint32_t)RESET) != (SYSCFG_CPSCTL & SYSCFG_CPSCTL_CPS_RDY)){
		return SET;
	}else{
		return RESET;
	}
}
